//******************************************************
//     **** ADC ROUTINES - SOURCE FILE *****
//******************************************************
//Controller		: ATmega32 (Clock: 16 Mhz)
//Compiler			: AtmelStudio 7
//Project Version	: 2.0
//Author			: CC Dharmani, Pune (India)
//					  www.dharmanitech.com
//Date				: June 2018
//******************************************************

#ifndef F_CPU
#define F_CPU 16000000UL		//Clock freq 16MHz
#endif

#include <avr/io.h>
#include "ADC_routines.h"


//******************************************************
//Purpose : Initialize the ADC
//Conversion time: 52uS
//******************************************************
void adcInit(void)
{
  ADCSRA = 0x00; //disable adc
  ADMUX  = 0x40;  //select adc input 0, ref:AVCC
  ADCSRA = 0x82; //prescaler:4, single conversion mode
  ADC_ENABLE;
}


//********************************************************************
//Purpose : Do an Analog to Digital Conversion for a given channel
//Paramtr :	8-bit channel number
//return  : float voltage value
//********************************************************************
float adcReadVolt(uint8_t channel)
{
    uint8_t i;
    uint16_t ADC_temp, ADCH_temp, ADC_var;
    float voltage;
	
	ADMUX = 0x40 | channel;			//select ADC channel
	ADC_var = 0;            
    for(i=0;i<4;i++)				//do the ADC conversion 4 times for better accuracy 
    {
	 	ADC_START_CONVERSION;
        while(!(ADCSRA & 0x10));	//wait for conversion done, ADIF flag active
        ADCSRA|=(1<<ADIF);
		
        ADC_temp = ADCL;			//read out ADCL register
        ADCH_temp = ADCH;			//read out ADCH register        
		ADC_temp +=(ADCH_temp << 8);
        ADC_var += ADC_temp;		//accumulate result (8 samples) for later averaging
    }

    ADC_var = ADC_var >> 2;			//average the 4 samples
	if(ADC_var > 1023) ADC_var = 1023;
	
	voltage = (float)ADC_var * ADC_REF / 1023.0;
	
    return (voltage);
}