//*******************************************************
//  *** ROUTINES FOR LCD DISPLAY - 4 bit interface ***
//*******************************************************
//Controller		: ATmega32 (Clock: 16 Mhz)
//Compiler			: AtmelStudio 7
//Project Version	: 2.0
//Author			: CC Dharmani, Pune (India)
//					  www.dharmanitech.com
//Date				: June 2018
//********************************************************

#ifndef F_CPU
#define F_CPU 16000000UL		//Clock freq 16MHz
#endif

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "LCD_4bit.h"

//***************************************
// short delay
//***************************************
 
void wait(void)
{
 uint8_t i;
 for(i=0;i<10;i++);
} 

//*************************************** 
// *** Initialize the LCD driver *** 
//***************************************
void LCD_init(void)
{
	SET_LCD_CMD;				// Set LCD to command

	LCD_Write4bits (0x30);      
	_delay_ms(10);
	LCD_Write4bits (0x30);
	_delay_ms(10);
	LCD_Write4bits (0x30);
	_delay_ms(10);
	LCD_Write4bits (0x20);
	_delay_ms(10);
	
	LCD_WriteCommand (0x2c);  	//4 bit interface
	_delay_ms(10);
	LCD_WriteCommand (0x08);	//display OFF
	_delay_ms(10);
	LCD_WriteCommand (0x01);	//clear LCD memory
	_delay_ms(10);
	LCD_WriteCommand (0x06);	//increment mode
	_delay_ms(10);
	LCD_WriteCommand (0x0c);	//display ON, cursor ON, blinking
	_delay_ms(10);
}

//*************************************************
// ** This routine will write one nibble to LCD **
//************************************************* 
void LCD_Write4bits(uint8_t c)
{
    c &= 0xf0;
    
	LCD_PORT &= 0x0f;	
	LCD_PORT |= c;		// Load data to port
    
	ENABLE_LCD;	   		// Write data to LCD
	asm("nop");
	asm("nop");
	DISABLE_LCD;	   	// Disable LCD
}

//**************************************************
//  *** Write a command instruction to the LCD ***
//**************************************************

void LCD_WriteCommand (uint8_t CMD)
{
	SET_LCD_CMD;				// Set LCD to command

	LCD_Write4bits(CMD);		// Load higher nibble to port
	wait();						// wait for LCD busy
	LCD_Write4bits(CMD << 4);	// Load lower nibble to port
	
	_delay_ms(1);
}


//**********************************************
//  *** Write one byte of data to the LCD ***
//**********************************************

void LCD_WriteData (uint8_t Data)
{
	SET_LCD_DATA;				// Set LCD to data
	
	LCD_Write4bits(Data);		// Load higher nibble to port
	wait();						// wait for LCD busy
	LCD_Write4bits(Data << 4);	// Load lower nibble to port
	
	_delay_ms(1);
}


//*********************************************************************
// ** Display a string at the specified row and column, using FLASH **
//*********************************************************************

void LCD_DisplayString_F (uint8_t row, uint8_t column, const char *string)
{
	LCD_Cursor (row, column);
	while (pgm_read_byte(&(*string)))
		LCD_WriteData(pgm_read_byte(&(*string++)));
}

//********************************************************************* 
// ** Display a string at the specified row and column, using FLASH ** 
//*********************************************************************

void LCD_DisplayString (uint8_t row, uint8_t column, char *string)
{
	LCD_Cursor (row, column);
	while (*string)
		LCD_WriteData(*string++);
}


//*********************************************************
//   *** Position the LCD cursor at "row", "column". ***
//*********************************************************
void LCD_Cursor (uint8_t row, uint8_t column)
{
	switch (row) {
		case 1: LCD_WriteCommand (0x80 + column - 1); break;
		case 2: LCD_WriteCommand (0xc0 + column - 1); break;
		//case 3: LCD_WriteCommand (0x94 + column - 1); break;  //case 3 & 4 are required only when LCD has 4 rows
		//case 4: LCD_WriteCommand (0xd4 + column - 1); break;
		default: break;
	}
}



