//*************************************************************
//*** FUNCTIONS FOR EEPROM READ/WRITE ON I2C COMMUNICATION  ***
//*************************************************************
//Controller		: ATmega32 (Clock: 16 Mhz)
//Compiler			: AtmelStudio 7
//Project Version	: 2.0
//Author			: CC Dharmani, Pune (India)
//					  www.dharmanitech.com
//Date				: June 2018
//*************************************************************

#ifndef F_CPU
#define F_CPU 16000000UL		//Clock freq 16MHz
#endif

#include <avr/io.h>
#include <avr/pgmspace.h>
#include "RTC_routines.h"

//***************************************************************************
//Function to set initial address of the RTC for subsequent reading / writing
//***************************************************************************
uint8_t RTC_setStartAddress(void)
{
   uint8_t errorStatus;
   
   errorStatus =i2c_start();
   if(errorStatus == 1)
   {
   	 i2c_stop();
	 return 1;
   } 
   
   errorStatus = i2c_sendAddress(DS1307_W);
   
   if(errorStatus == 1)
   {
	 i2c_stop();
	 return 1;
   } 
   
   errorStatus = i2c_sendData(0x00);
   if(errorStatus == 1)
   {
	 i2c_stop();
	 return 1;
   } 

   i2c_stop();
   return 0;
}

//***********************************************************************
//Function to read RTC registers and store them in buffer rtc_register[]
//***********************************************************************    
uint8_t RTC_read(void)
{

  uint8_t errorStatus, i, data;
  
  errorStatus = i2c_start();
   if(errorStatus == 1)
   {
   	 i2c_stop();
	 return 1;
   } 
   
   errorStatus = i2c_sendAddress(DS1307_W);
   
   if(errorStatus == 1)
   {
	 i2c_stop();
	 return 1;
   } 
   
   errorStatus = i2c_sendData(0x00);
   if(errorStatus == 1)
   {
	 i2c_stop();
	 return 1;
   } 

    errorStatus = i2c_repeatStart();
   if(errorStatus == 1)
   {
   	 i2c_stop();
	 return 1;
   } 
   
    errorStatus = i2c_sendAddress(DS1307_R);
   
   if(errorStatus == 1)
   {
	 i2c_stop();
	 return 1;
   } 
 
    for(i=0;i<7;i++)
   {
      if(i == 6)  	 //no Acknowledge after receiving the last byte
	   	  data = i2c_receiveData_NACK();
	  else
	  	  data = i2c_receiveData_ACK();
		  
   	  if(data == ERROR_CODE)
   	  {
			i2c_stop();
	   		return 1;
   	  }
	  
	  rtc_register[i] = data;
	}
	
	i2c_stop();
	return 0;
}	  

//******************************************************************
//Function to write new time in the RTC 
//******************************************************************   
uint8_t RTC_writeTime(void)
{
  uint8_t errorStatus, i;
  
   errorStatus = i2c_start();
   if(errorStatus == 1)
   {
   	 i2c_stop();
	 return(1);
   } 
   
   errorStatus = i2c_sendAddress(DS1307_W);
   
   if(errorStatus == 1)
   {
	 i2c_stop();
	 return(1);
   } 
   
   errorStatus = i2c_sendData(0x00);
   if(errorStatus == 1)
   {
	 i2c_stop();
	 return(1);
   } 

    for(i=0;i<3;i++)
    {
	  errorStatus = i2c_sendData(rtc_register[i]);  
   	  if(errorStatus == 1)
   	  {
			i2c_stop();
	   		return(1);
   	  }
    }
	
	i2c_stop();
	return(0);
}


//******************************************************************
//Function to write new date in the RTC
//******************************************************************   
uint8_t RTC_writeDate(void)
{
  uint8_t errorStatus, i;
  
   errorStatus = i2c_start();
   if(errorStatus == 1)
   {
   	 i2c_stop();
	 return(1);
   } 
   
   errorStatus = i2c_sendAddress(DS1307_W);
   
   if(errorStatus == 1)
   {
	 i2c_stop();
	 return(1);
   } 
   
   errorStatus = i2c_sendData(0x03);
   if(errorStatus == 1)
   {
	 i2c_stop();
	 return(1);
   } 

    for(i=3;i<7;i++)
    {
	  errorStatus = i2c_sendData(rtc_register[i]);  
   	  if(errorStatus == 1)
   	  {
			i2c_stop();
	   		return(1);
   	  }
    }
	
	i2c_stop();
	return(0);
}

//******************************************************************
//Function to get RTC date & time in FAT32 format
//******************************************************************   
/*
unsigned char getDateTime_FAT(void)
{
  
   unsigned char mth, dt, hr, min, sec, error; 
   unsigned int yr;

   error = RTC_read();
   if(error) return 1;

   yr = (YEAR & 0xf0) >> 4;
   yr = (yr * 10)+(YEAR & 0x0f);
   yr = yr+2000;
   yr = yr - 1980;

   dateFAT = yr;

   mth = (MONTH & 0xf0) >> 4;
   mth = (mth * 10)+(MONTH & 0x0f);

   dateFAT = (dateFAT << 4) | mth;

   dt = (DATE & 0xf0) >> 4;
   dt = (dt * 10)+(DATE & 0x0f);

   dateFAT = (dateFAT << 5) | dt;


   hr = (HOURS & 0xf0) >> 4;
   hr = (hr * 10)+(HOURS & 0x0f);

   timeFAT = hr;

   min = (MINUTES & 0xf0) >> 4;
   min = (min * 10)+(MINUTES & 0x0f);

   timeFAT = (timeFAT << 6) | min;

   sec = (SECONDS & 0xf0) >> 4;
   sec = (sec * 10)+(SECONDS & 0x0f);
   sec = sec / 2;    //FAT32 fromat accepts dates with 2sec resolution (e.g. value 5 => 10sec)

   timeFAT = (timeFAT << 5) | sec;

   
   return 0;
}
*/