//**************************************************************
//******** SERIAL COMMUNICATION UART ROUTINES *******
//**************************************************************
//Controller		: ATmega32 (Clock: 16 Mhz)
//Compiler			: AtmelStudio 7
//Project Version	: 2.0
//Author			: CC Dharmani, Pune (India)
//					  www.dharmanitech.com
//Date				: Jun 2018
//**************************************************************

#ifndef F_CPU
#define F_CPU 16000000UL		//Clock freq 16MHz
#endif

#include <stdio.h>
#include <avr/io.h>
#include "UART_routines.h"

//**************************************************
//UART0 initialize
//char size: 8 bit
//parity: Disabled
//**************************************************
void uartInit(void)
{
	UBRRH = (uint8_t)(UBRR_VALUE >> 8);
	UBRRL = (uint8_t)UBRR_VALUE;
	UCSRC = (1 << URSEL) | (1 << UCSZ1) | (1 << UCSZ0);
	UCSRB = (1 << RXEN) | (1 << TXEN);
}

//***************************************************
//Function to transmit a single byte
//***************************************************
int transmitByte(char data, FILE *stream)
{
	if(data == '\n') transmitByte('\r', 0);
	
	while(!(UCSRA&(1<<UDRE))); //wait while previous byte is completed
	
	UDR = data;	// Transmit data
	return 0;
}

//*************************************************
//Function to receive a single byte
//*************************************************
int receiveByte(FILE *stream)
{
	uint8_t data;
	
	while(!(UCSRA & (1 << RXC)));	// Wait for byte to be received
	data = UDR;
	
	transmitByte(data,stream); //echo back received data
	
	return data;
}



//************ END ***** www.dharmanitech.com *******
