//**************************************************************
//   ******** FUNCTIONS FOR DS1621 TEMP SENSOR  *******
//**************************************************************
//Controller		: ATmega32 (Clock: 16 Mhz)
//Compiler			: AtmelStudio 7
//Project Version	: 2.0
//Author			: CC Dharmani, Pune (India)
//					  www.dharmanitech.com
//Date				: June 2018
//**************************************************************

#ifndef F_CPU
#define F_CPU 16000000UL		//Clock freq 16MHz
#endif

#include <avr/io.h>
#include <util/delay.h>
#include "ds1621.h"

//******************************************************************
//Function to send command
//******************************************************************    
void ds1621_init(void)
{
	uint8_t errorStatus;
  
	errorStatus = i2c_start();
	if(errorStatus == 1)
	{
		//LCD_writeString_F("i2c start failed..");
   		i2c_stop();
		return;
	} 
   
	errorStatus = i2c_sendAddress(DS1621_W);
   
	if(errorStatus == 1)
	{
		//LCD_writeString_F("ds1621 sendAddress failed..");
		i2c_stop();
		return;
	} 
   
	errorStatus = i2c_sendData(ACCESS_CONFIG);
	if(errorStatus == 1)
	{
		//LCD_writeString_F("EEPROM write-2 failed..");
		i2c_stop();
		return;
	} 
   
	errorStatus = i2c_sendData(0x03);   //commmand to set o/p polarity high, single shot conversion
	if(errorStatus == 1)
	{
		//LCD_writeString_F("EEPROM write-2 failed..");
		i2c_stop();
		return;
	} 
	i2c_stop();
}

//******************************************************************
//Function to send command
//******************************************************************    
void ds1621_sendCommand(uint8_t command)
{
	uint8_t errorStatus;
  
	errorStatus = i2c_start();
	if(errorStatus == 1)
	{
		//LCD_writeString_F("i2c start failed..");
   		i2c_stop();
		return;
	} 
   
	errorStatus = i2c_sendAddress(DS1621_W);
   
	if(errorStatus == 1)
	{
		//LCD_writeString_F("ds1621 sendAddress failed..");
		i2c_stop();
		return;
	} 
   
	errorStatus = i2c_sendData(command);
	if(errorStatus == 1)
	{
		//LCD_writeString_F("EEPROM write-2 failed..");
		i2c_stop();
		return;
	} 

	i2c_stop();
}

//******************************************************************
//Function to read 
//******************************************************************    

uint8_t ds1621_readValue(uint8_t value)
{
	uint8_t errorStatus, data;
  
	errorStatus = i2c_start();
	if(errorStatus == 1)
	{
		//LCD_writeString_F("RTC start1 failed..");
   		i2c_stop();
		return (0);
	} 
   
	errorStatus = i2c_sendAddress(DS1621_W);
   
	if(errorStatus == 1)
	{
		//LCD_writeString_F("ds1621 sendAddress1 failed..");
		i2c_stop();
		return (0);
	} 
   
	errorStatus = i2c_sendData(value);
	if(errorStatus == 1)
	{
		//LCD_writeString_F("ds1621 write failed..");
		i2c_stop();
		return (0);
	} 

	errorStatus = i2c_repeatStart();
	if(errorStatus == 1)
	{
		//LCD_writeString_F("ds1621 repeat start failed..");
   		i2c_stop();
		return (0);
	} 
   
	errorStatus = i2c_sendAddress(DS1621_R);
   
	if(errorStatus == 1)
	{
		//LCD_writeString_F("ds1621 sendAddress2 failed..");
		i2c_stop();
		return (0);
	} 
 
	data = i2c_receiveData_NACK(); 
   
	i2c_stop();
   
	return (data);
}	  

//******************************************************************
//Function to read 
//******************************************************************    
float getTemperature(void)
{
	int8_t temperature;
	int8_t counter, slope;
	float actualTemp;
	
	ds1621_sendCommand(START_CONVERT);			//Temp. conversion takes place in 750ms (max) after this command
												//hence the registers read immediately after this command, output the 
												//results of previously sent START_CONVERT command
												//if current reading is required, then add _delay_ms(750) after the command
	temperature = ds1621_readValue(READ_TEMP);
	counter = ds1621_readValue (READ_COUNTER);
	slope = ds1621_readValue (READ_SLOPE);
	actualTemp = (float)temperature - 0.25 + ((float)(slope - counter) / (float)slope);
	
	return actualTemp;
}