//**************************************************************
// ****  Main - Header File  ****
//**************************************************************
//Controller		: ATmega32 (Clock: 16 Mhz)
//Compiler			: AtmelStudio 7
//Project Version	: 3.0
//Author			: CC Dharmani, Pune (India)
//					  www.dharmanitech.com
//Date				: June 2018, Updated Sep 2021
//**************************************************************

#ifndef _MAIN_H_
#define _MAIN_H_

#define INTERVAL 1000	//interval in milliseconds, between two measurements (in multiples of 10)
						//minimum: 50; maximum: 3600000 (i.e. 3600 seconds or 1 hour)
						//For intervals less than 100, 1 or 2 samples may be missing for every 1 sec time, due to the display/RTC updates

#define MAX_TICK_VALUE	4147200000UL	//tick_1ms timer maximum limit is set to 48 days, for resetting it
										//before it actually overflows after 49 days (2^32 milliseconds)

//Comment out any one of the below macros if you don't need both the logging
//#define UART_LOGGING_ENABLED	1
#define SD_LOGGING_ENABLED		1

#define START_TIMER			TCCR2 = 0x03
#define STOP_TIMER			TCCR2 = 0x00
#define RESET_TIMER			TCNT2 = 0x06; tick_1ms = 0

#define GREEN_LED_ON		PORTD |= 0x40
#define GREEN_LED_OFF		PORTD &= ~0x40
#define RED_LED_ON			PORTD |= 0x80
#define RED_LED_OFF			PORTD &= ~0x80

#define SET_PRESSED			!(PIND & 0x08)
#define UP_PRESSED			!(PIND & 0x10)
#define DWN_PRESSED			!(PIND & 0x20)
#define ANY_BUTTON_PRESSED	((PIND & 0x38) != 0x38)

void port_init(void);
void timer0_init(void);
void init_devices(void);
void updateDisplay(void);
void buttonResponseRTC(void);
void blinkRedLED(void);

#endif 